function Model_Surface_And_Get_Distances(aImarisApplicationID)
%
%
%  Copyright Chris Barilla, University of Rochester, 2017
%
%
%  Installation:
%  - Copy this file into the XTensions folder in the Imaris installation directory.
%  - You will find this function in the Image Processing menu
%
%    <CustomTools>
%      <Menu>
%        <Item name="Model_Surface_And_Get_Distances" icon="Matlab" tooltip="Finds distances between tracks in two chosen surface items.">
%          <Command>MatlabXT::Model_Surface_And_Get_Distances(%i)</Command>
%        </Item>
%      </Menu>
%    </CustomTools>
%
%   Description:
%   This function will find the distance between surface objects. This
%   function was developed for a particular situation. The situation is when
%   cells are trakced along with a fixed surface. The purpose of the XTension
%   is to find the distance from the mask of the fixed surface to the center
%   of the cell surfaces. The distance is measured for each timepoint. There
%   is also an option to make a model of the fixed surface. This is because
%   the surface that we were originally tracking had an inconsistent signal
%   that left large gaps in the native Imaris tracking. But by using matlab
%   functions we were able to make a model of the surface so that whole
%   imaging area is covered. There is a verbose option that gives the
%   measurement points from the cells to the fixed surface. 
%
%


        fprintf('\n\tStarting XT...\n');
        
        tic;
        
        w=warning('off','all');

	% connect to Imaris interface
		vImarisApplication = ConnectToImaris( aImarisApplicationID );
		vImarisDataSet = vImarisApplication.GetDataSet;
	
        
    % the following is from Henry Pinkard's Imaris extension
	% preparing parameters for the distance transform
		vDataMin = [vImarisDataSet.GetExtendMinX, vImarisDataSet.GetExtendMinY, vImarisDataSet.GetExtendMinZ];
		vDataMax = [vImarisDataSet.GetExtendMaxX, vImarisDataSet.GetExtendMaxY, vImarisDataSet.GetExtendMaxZ];
		vDataSize = [vImarisDataSet.GetSizeX, vImarisDataSet.GetSizeY, vImarisDataSet.GetSizeZ];
		pixelSizeZ = (vDataMax(3) - vDataMin(3)) / vDataSize(3);
		pixelSizeX = (vDataMax(1) - vDataMin(1)) / vDataSize(1);
		pixelSizeY = (vDataMax(2) - vDataMin(2)) / vDataSize(2);
        aspect = [pixelSizeX pixelSizeY pixelSizeZ];
		numTimePoints = vImarisDataSet.GetSizeT;
	
        
    % choose Surfaces object with tracked cells
        [CellSurfaces vObjectType] = chooseObjects( vImarisApplication, 'Choose Surface with Tracked Cells:');

        
    % MakeModelSurface creates a new Imaris Surface item from a reference
    % surface
        fprintf('Fitting Model to Surface (Part 1 of 2)\n');
        [models] = promptModelSurface(vImarisApplication);  
        
    % starts progress bar
        fprintf('Calculating Distances (Part 2 of 2)\n');
        
    % distances for each model created
        for i = 1:length(models)
            %ModelSurface = models(i).ModelSurface;
            model_xyzs = models(i).model_xyzs;
            layer_type = models(i).layer_type;
            getDistancesFromModeledSurf(vImarisApplication, CellSurfaces, model_xyzs, layer_type);
        end

        
        fprintf('Done\n')	

        disp('Run Time:')
        toc

        hold on;

end


function [xc,yc,R,a] = circfit(x,y)
%
%   [xc yx R] = circfit(x,y)
%
%   fits a circle  in x,y plane in a more accurate
%   (less prone to ill condition )
%  procedure than circfit2 but using more memory
%  x,y are column vector where (x(i),y(i)) is a measured point
%
%  result is center point (yc,xc) and radius R
%   an optional output is the vector of coeficient a
%   describing the circle's equation
%
%   x^2+y^2+a(1)*x+a(2)*y+a(3)=0
%
%  By:  Izhak bucher 25/oct /1991, 
    x = x(:); 
    y = y(:);
    a = [x y ones(size(x))]\[-(x.^2+y.^2)];
    xc = -.5*a(1);
    yc = -.5*a(2);
    R  =  sqrt((a(1)^2+a(2)^2)/4-a(3));
   
end


function getMinDistanceBetweenModels(vImarisApplication, models, aspect)

        mask1 = models(1).masks;
        mask1 = mask1{1};
        mask2 = models(2).masks;
        mask2 = mask2{1};
    	mask1 = mask1(:,:,:,models(1).chosen_channel,1);
    	mask2 = mask2(:,:,:,models(2).chosen_channel,1);

        vals = zeros(size(mask1,1),size(mask1,2));
        between_data  = null(1);
        
        
        for i=1:size(mask1,1)
            for j=1:size(mask1,2)
                z = find(mask2(i,j,:));
                coord = [i j z] .* aspect;
                [min_d,min_coord1,min_coord2] = getMinDistanceOfMasksAndCoord(mask1,coord,aspect);
                vals(i,j) = min_d;
                if i==1 && j==1
                    min_dist = min_d;
                    min_dist_coord1 = min_coord1;
                    min_dist_coord2 = min_coord2;
                    max_dist = min_d;
                    max_dist_coord1 = min_coord1;
                    max_dist_coord2 = min_coord2;
                end
                if min_d < min_dist
                    min_dist = min_d;
                    min_dist_coord1 = min_coord1;
                    min_dist_coord2 = min_coord2;
                end
                if min_d > max_dist
                    max_dist = min_d;
                    max_dist_coord1 = min_coord1;
                    max_dist_coord2 = min_coord2;
                end
            end
        end   
        
        mean_val = mean(mean(vals));
        temp = abs(vals-mean_val);
        min_val = min(min(temp));
        ind = find(temp == min_val);
        [i, j] = ind2sub(ind);
        z = find(mask2(i,j,:));
        coord = [i j z] .* aspect;
        [~,mean_coord1,mean_coord2] = getMinDistanceOfMasksAndCoord(mask1,coord,aspect);
        
        
        
        
        between_data = addMeasurementPointData(between_data, min_dist_coord1, min_dist_coord2, 'min', ' ', 0,0);
        between_data = addMeasurementPointData(between_data, max_dist_coord1, max_dist_coord2, 'max', ' ', 0,0);
        between_data = addMeasurementPointData(between_data, mean_coord1, mean_coord2, '~mean', ' ', 0,0);

        measurementName = ['Distance Between Models'];
        setMeasurementPoint(vImarisApplication, measurementName, between_data, 0.9, 0.3)
        
        min_str = ['minimum distance between models: ' num2str(min_dist)];
        max_str = ['maximum distance: ' num2str(max_dist)];
        mean_str = ['mean distance: ' num2str(mean_val)];
        
        
        msgbox( {min_str;max_str;mean_str} )
end


function setMeasurementPoint(vImarisApplication, name, data, vRed, vGreen)
    
            newMP = vImarisApplication.GetFactory.CreateMeasurementPoints();
            measurementPositions = cell2mat(data(:,[1:3])); 
            measurementTimes = cell2mat(data(:,4));
            measurementNames = data(:,5);
            newMP.Set(measurementPositions, measurementTimes, measurementNames);
            %vRed  = 
            %vGreen = 0.3;
            vBlue = 0.1;vAlpha = 0.0;vRGBA = [vRed, vGreen, vBlue, vAlpha];
            vRGBA = round(vRGBA * 255);vRGBA = uint32(vRGBA * [1; 256; 256*256; 256*256*256]); % combine different components (four bytes) into one integer
            newMP.SetColorRGBA (vRGBA);
            newMP.SetName (name);
            newMP.SetVisible (1);
            vSurpassScene = vImarisApplication.GetSurpassScene;
            vSurpassScene.AddChild(newMP, -1);
end


function [measurement_data] = addMeasurementPointData(measurement_data, coord1, coord2, name1, name2, time1, time2)

    if size(measurement_data,1) == 1
        indx2 = 0;
    else
        indx2 = size(measurement_data,1);
    end

    indx2 = indx2 + 1;
    measurement_data{indx2,1} = coord1(1); 
    measurement_data{indx2,2} = coord1(2); 
    measurement_data{indx2,3} = coord1(3); 
    measurement_data{indx2,4} = time1;
    measurement_data{indx2,5} = name1; % surfaceIds(surfaceIndex);

    indx2 = indx2 + 1;
    measurement_data{indx2,1} = coord2(1); 
    measurement_data{indx2,2} = coord2(2); 
    measurement_data{indx2,3} = coord2(3);
    measurement_data{indx2,4} = time2;
    measurement_data{indx2,5} = name2; % ' ';
end


function res = myrepelem( mat, siz )
    
    res = zeros(size(mat,1)*siz, 3);
    ind = 1;
    for i = 1:size(mat,1)
       res((ind):(ind+siz-1),:) = repmat(mat(i,:), siz, 1);
       ind = ind + siz;
    end

end


function [min_distance,min_coord1,min_coord2] = getMinDistanceOfMasksAndCoord(mask1,coord,aspect)
    %
    %   fastest distance function
    %
    inds1 = find(mask1);
    
    [x, y, z] = ind2sub(size(mask1),inds1);
    coord1 = [x y z] .* repmat( aspect, length(x), 1);
    coord2 = repmat( coord, length(x), 1);
    distances = sqrt(sum((coord1 - coord2) .^ 2,2));
    [min_distance, min_indx] = min(distances);
    min_coord1 = coord1(min_indx,:);
    min_coord2 = coord;
end


function [min_distance,min_coord1,min_coord2] = getMinDistanceOfMasks(mask1,mask2,aspect)
    %
    %   fastest distance function
    %
    inds1 = find(mask1);
    inds2 = find(mask2);
    
    [x, y, z] = ind2sub(size(mask1),inds1);
    coord1 = [x y z] .* repmat( aspect, length(x), 1);
    coord1 = myrepelem(coord1, length(inds2));
    [x, y, z] = ind2sub(size(mask2),inds2);
    coord2 = [x y z] .* repmat( aspect, length(x), 1);
    coord2 = repmat( coord2, length(inds1), 1);
    distances = sqrt(sum((coord1 - coord2) .^ 2,2));
    [min_distance, min_indx] = min(distances);
    min_coord1 = coord1(min_indx,:);
    min_coord2 = coord2(min_indx,:);
    
end


function AddStats( vObject, StatName, unit, surfaceTimeIndices, surfaceIds, values  )

%
% 	AddStats adds parameter to statistic output of Imaris
%
%       @param vObject
%       @param surfaceName
%       @param numSurfaces
%       @param surfaceTimeIndices
%       @param surfaceIds
%       @param values
%
        numSurfaces = length(values);

		%StatName = ['Distance From ' char(surfaceName) ];
		Names = repmat({StatName}, [numSurfaces 1]); 

        % Remove stats with same name
        vObject.RemoveStatistics(Names);
        
		% Create the stat unit list.
		Units = repmat({unit}, [numSurfaces, 1]); 

		% Assemble the factors cell array.
		Factors = cell(3, numSurfaces);

		% Set the Category.
		Factors(1, :) = repmat({'Surface'}, [numSurfaces 1]);

		% Set the Collection to an empty string. 
		Factors(2, :) = repmat({''}, [numSurfaces 1]);

		% Set the Time and Track ID
		Factors(3, :) = num2cell(surfaceTimeIndices + 1);
		
		% Convert the numbers to strings.
		Factors(3, :) = cellfun(@num2str, Factors(3, :), 'UniformOutput', 0);

		% Create the factor names.
		factorNames = {'Category', 'Collection', 'Time'};

		% Create the stat IDs.
		IDs = transpose(surfaceIds);

		% Send the distance statistics to Imaris.
		vObject.AddStatistics(Names, values, Units, ...
        	Factors, factorNames, IDs)

end


function tracksStruct = getTrackIndicesStruct( vObject )
%
%	getTrackIndicesStruct function creates a structure that contains the track IDs and their associated surface IDs
%	
%		@param vObject - The ISurfaces object to be parsed for track information
%

	% gets track and surface id info from ISurface Object
    trackids = int32(vObject.GetTrackIds);
	trackedges = vObject.GetTrackEdges;
	tracks = [trackids trackedges];
	
    unqiueTrackIDs = unique(trackids);
    objectIDs = vObject.GetIds();
     
    tracksStruct(length(unqiueTrackIDs)) = struct();
    
    % iterate through trackIDs getting the associated surface ids
    for i = 1:length(unqiueTrackIDs)
        
        inds = tracks(:,1) == unqiueTrackIDs(i);
        tracksi = tracks(inds,:);
     
        inds = [tracksi(1:end, 2) tracksi(1:end, 3)].';         
        inds = unique(inds);      
        
        TrackIDs = tracksi(1:size(tracksi,1), 1);
        TrackIDs = [transpose(TrackIDs) TrackIDs(1)].';

        ids = objectIDs(inds+1);
        
        times = nan( length(inds), 1 );
        coords = nan( length(inds), 3 );

        for j = 1:size(inds,1)
            ind = inds(j);
            times(j) = vObject.GetTimeIndex(ind);
            coords(j,:) = vObject.GetCenterOfMass(ind);
        end
               
        % add Surface IDs and Track ID to structure
        tracksStruct(i).TrackID = unqiueTrackIDs(i);
        tracksStruct(i).TrackIDs = TrackIDs;
        tracksStruct(i).ids = ids;
        tracksStruct(i).inds = inds;
 		tracksStruct(i).times = times;
 		tracksStruct(i).coords = coords;
    end
    
end


function [vObject, objectType] = chooseObjects( vImarisApplication, prompt, hack, varargin )

%
% 	chooseObjects prompts the user to choose a Surface or Spot Item in the Surpass Scene
%
%		@param vImarisApplication - The application index number given by Imaris
%		@param prompt - string that gives the message to be displayed for user
%

	vSurpassScene = vImarisApplication.GetSurpassScene;
	nObjects=0;

    for vChildIndex = 1:vSurpassScene.GetNumberOfChildren
        vDataItem = vSurpassScene.GetChild(vChildIndex - 1);
        if vImarisApplication.GetFactory.IsSurfaces(vDataItem)
            nObjects=nObjects+1;
            vObjects{nObjects} = vImarisApplication.GetFactory.ToSurfaces(vDataItem);    
            ObjectNames{nObjects}=char(vObjects{nObjects}.GetName);
            ObjectType{nObjects}='Surface';
        elseif vImarisApplication.GetFactory.IsSpots(vDataItem)
            nObjects=nObjects+1;
            vObjects{nObjects} = vImarisApplication.GetFactory.ToSpots(vDataItem);    
            ObjectNames{nObjects}=char(vObjects{nObjects}.GetName);
            ObjectType{nObjects}='Spot';
        end
    end 

    if nargin == 3
        choice = hack;
    else
        choice = menu(prompt,ObjectNames);
    end
    vObject = vObjects{choice};
    objectType = ObjectType{choice}; 
    
%     disp( objectType);
%     disp (vObject);

end


function vImarisApplication = ConnectToImaris( aImarisApplicationID )
%
% 	ConnectToImaris connects matlab to the Imaris software portal. 
%
%		@param vImarisApplication - The application index number given by Imaris
%
	if ~isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
	  javaaddpath ImarisLib.jar
	  vImarisLib = ImarisLib;
	  if ischar(aImarisApplicationID)
		aImarisApplicationID = round(str2double(aImarisApplicationID));
	  end
	  vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
	else
	  vImarisApplication = aImarisApplicationID;
	end
	
end


function addTrackMean( vObject, statName, unit, distances, surfaceTrackIDs )

    uniqueTrackIds = unique(surfaceTrackIDs);
    numTracks = numel(uniqueTrackIds);
    meanVals = zeros(numTracks,1);
    
    for tid = 1:numTracks
        inds = surfaceTrackIDs == uniqueTrackIds(tid);
        vals = distances(inds);
        meanVals(tid) = mean(vals);
    end
    
    %BaseName = ['Test Track Distance From ' char(surfaceName) ];
    Names = repmat({statName}, [numTracks 1]); 

    vObject.RemoveStatistics(Names);

    % Create the stat unit list.
    Units = repmat({unit}, [numTracks, 1]); 

    % Assemble the factors cell array.
    Factors = cell(2, numTracks);

    % Set the Category.
    Factors(1, :) = repmat({'Surface'}, [numTracks 1]);

    % Set the Collection to an empty string. 
    Factors(2, :) = repmat({''}, [numTracks 1]);

    % Create the factor names.
    factorNames = {'Category', 'Collection'};

    % Create the stat IDs.
    IDs = transpose(uniqueTrackIds);

    % Send the distance statistics to Imaris.
    vObject.AddStatistics(Names, meanVals, Units, ...
        Factors, factorNames, IDs)

end


function [intensity_threshold, max_z_threshold, min_z_threshold, chosen_channel, top_layer] = threshold_intensity_and_depth(data, layer, channel, varargin)
%
%   gui_threshold will get the z-axis and intensity threshold to pinpoint
%   the image signal to model
%

    if nargin == 3
        top_layer = layer;
        chosen_channel = channel;
    elseif nargin == 2
        chosen_channel = choose_channel(data);
        top_layer = layer;
    else
        chosen_channel = choose_channel(data);
        top_layer = 1;
    end
%     chosen_channel = choose_channel_2(data);
%     chosen_channel = 1;

    % graph image with default thresholds
    [mx,my,mz,mc,mt]=size(data);
    cmap=[1:mc];
    background=mean(data(:,:,:,chosen_channel,:),5);
    min_intensity = min(min(min(min(min(data)))));
    max_intensity = max(max(max(max(max(data)))));
    f = figure;
    intensity_threshold = mean([min_intensity max_intensity]);
%    intensity_threshold = 600;
    max_z_threshold = mz;
    min_z_threshold = 1;
    [xxx,yyy,zzz]=meshgrid([1:my],[1:mx],fliplr([1:mz]));
    bwbackground = background > intensity_threshold;
    

    if top_layer
        for i=1:mx
            for j=1:my
                k=find(bwbackground(i,j,:)>0,1);
                bwbackground(i,j,k+1:end)=0;
            end
        end
    else
        for i=1:mx
            for j=1:my
                k=find(bwbackground(i,j,:)>0);
                maxk = max(k);
                bwbackground(i,j,1:maxk-1)=0;
            end
        end     
    end
        

    
    isosurface(xxx,yyy,zzz,bwbackground,[.9])
    axis ij;
    axis equal;  

    % Create slider figure
    S.fh = figure('units','pixels',...
        'position',[400 400 300 240],...
        'menubar','none',...
        'name','Choose intensity_threshold',...
        'numbertitle','off',...
        'resize','off',...
        'deletefcn',{@delete_call,f});

    % Create Sliders     
    S.sl_intensity = uicontrol('style','slide',...
        'unit','pixel',...
        'String','intensity_threshold',...
        'position',[10 30 200 20],...
        'min',min_intensity,'value',intensity_threshold,'max',max_intensity,...
        'callback',{@sl_call,background,f});

    S.sl_z_max_threshold = uicontrol('style','slide',...
        'unit','pixel',...
        'String','max_z_threshold',...
        'position',[230 10 30 200],...
        'min',1,'value',mz,'max',mz,...
        'callback',{@sl_call,background,f});

    S.sl_z_min_threshold = uicontrol('style','slide',...
        'unit','pixel',...
        'String','min_z_threshold',...
        'position',[260 10 30 200],...
        'min',1,'value',1,'max',mz,...
        'callback',{@sl_call,background,f});

    % Apply Button         
    S.button = uicontrol('style','pushbutton',...
        'unit','pixel',...
        'position',[160 5 60 25],...
        'fontsize',12,...
        'string','Apply',...
        'callback',{@delete_call,f});   

    
    % radial buttons
    
    S.bg = uibuttongroup(S.fh,'Visible','on',...
                  'unit','pixel',...
                  'Position',[5 120 180 100]);
    S.r1 = uicontrol(S.bg,'Style','radiobutton',...
                  'String','Get Top Layer',...
                  'unit','pixel',...
                  'Position',[5 60 120 30],...
                  'HandleVisibility','on',...
                  'callback',{@sl_call,background,f});              
    S.r2 = uicontrol(S.bg,'Style','radiobutton',...
                  'String','Get Bottom Layer',...
                  'unit','pixel',...
                  'Position',[5 30 120 30],...
                  'HandleVisibility','on',...
                  'callback',{@sl_call,background,f});
    
    if top_layer
        s.r1.Value = 1;
        s.r2.Value = 0;
    else
        s.r2.Value = 1;
        s.r1.Value = 0;
    end
    
    
    
    % Add slider labels
    S.text = uicontrol('style','text',...
        'unit','pixel',...
        'position',[10 55 200 30],...
        'fontsize',12,...
        'string',num2str(intensity_threshold)); 

    S.text2 = uicontrol('style','text',...
        'unit','pixel',...
        'position',[10 85 200 30],...
        'fontsize',12,...
        'string','Set Intensity Theshold:'); 

    S.text3 = uicontrol('style','text',...
        'unit','pixel',...
        'String','Set z-axis:',...
        'position',[200 213 100 20],...
        'fontsize',12);
             
                                  
    set(f,'deletefcn',{@delete_call,S.fh})  % Closing one closes the other.
    
    
    
    function [intensity_threshold, max_z_threshold, min_z_threshold] = sl_call(varargin) 
    % Callback for the slider.
        [h,background,f] = deal(varargin{[1;3;4;]});  % Get the calling handle and structure.
        
        % get dimensions of image
        [mx,my,mz,mc,mt]=size(background);

        % erase existing figure and prime it for new graph
        f.clo
        figure(f)
        
        % get threshold values
        intensity_threshold = get(S.sl_intensity,'val');
        max_z_threshold = ceil(get(S.sl_z_max_threshold,'val'));
        min_z_threshold = ceil(get(S.sl_z_min_threshold,'val'));
        % normalize the threshold values to correct oreintation
        max_z_threshold =  mz - max_z_threshold + 1;
        min_z_threshold = mz - min_z_threshold + 1; 

        
        % check if min is less than max threshold. If not, switch values
        temp_min = min([max_z_threshold min_z_threshold]);
        max_z_threshold = max([max_z_threshold min_z_threshold]);
        min_z_threshold = temp_min;

%         disp('mz')
%         disp(mz)
%         disp('min_z_threshold')
%         disp(min_z_threshold)
%         disp('max_z_threshold')
%         disp(max_z_threshold)
        
        % make isosurface using threshold values
        temp = zeros(size(background));
        temp(:,:,(min_z_threshold:max_z_threshold)) = background(:,:,(min_z_threshold:max_z_threshold));
        background  = temp;
        [xxx,yyy,zzz]=meshgrid([1:my],[1:mx],fliplr([1:mz]));
        
        
        set(S.text,'string',num2str(intensity_threshold));
        bwbackground = background > intensity_threshold;
        
        if S.r1.Value == 1 
            top_layer = 1;
        else
            top_layer = 0;
        end
        
        if top_layer
            for i=1:mx
                for j=1:my
                    k=find(bwbackground(i,j,:)>0,1);
                    bwbackground(i,j,k+1:end)=0;
                end
            end
        else
            for i=1:mx
                for j=1:my
                    k=find(bwbackground(i,j,:)>0);
                    maxk = max(k);
                    bwbackground(i,j,1:maxk-1)=0;
                end
            end     
        end
        
        
        
        disp('computing...');
        isosurface(xxx,yyy,zzz,bwbackground,[.9]);
        disp('computed');
        axis ij;
        axis equal;
    end


    function [] = delete_call(varargin)
    % Callback for the pushbutton.
        intensity_threshold = get(S.sl_intensity,'val');        
        max_z_threshold = ceil(get(S.sl_z_max_threshold,'val'));
        min_z_threshold = ceil(get(S.sl_z_min_threshold,'val'));
        max_z_threshold =  mz - max_z_threshold + 1;
        min_z_threshold = mz - min_z_threshold + 1; 
        temp_min = min([max_z_threshold min_z_threshold]);
        max_z_threshold = max([max_z_threshold min_z_threshold]);
        min_z_threshold = temp_min;
        if S.r1.Value == 1 
            top_layer = 1;
        else
            top_layer = 0;
        end
        delete(S.fh);  % Closes the GUI, allows the new R to be returned.
        delete(f);
    end

    uiwait(S.fh)
    
end


function chnnl = choose_channel(data)
% 
% will prompt user to select channel based off images
%   Wil only work with 6 or less channels
%
% Author: Chris Barilla
% Date:  3/23/2017

%% Set Screen
    clear zslice
    for i=1:size(data,5)
        zslice{i}=squeeze(uint16(mean(data(:,:,:,:,i),3))); %note mean returns a double so we need to convert it back to uint16
    end

    chnnl = 0;

    Pix_SS = get(0,'screensize');
    width = Pix_SS(3) / 2;
    height = Pix_SS(4) / 2;
    x = 10;
    y = Pix_SS(4) - height - 30;
    
    S.fh = figure('units','pixels',...
                  'position',[x y width height],...
                  'menubar','none',...
                  'name','Choose Channel',...
                  'numbertitle','off',...
                  'resize','on');

    numChannels = size(zslice{1},3);
    rows = ceil(numChannels/3);

    for i=1:numChannels
        channelsStr{i} = num2str(i);
        subplot(rows,3,i);
        imshow(imadjust(zslice{1}(:,:,i)));
    end

    
    S.prompt = uicontrol('Style', 'text',...
        'String', 'Select Channel to Register on:',...
        'FontSize',8 ,...
        'Position', [10, 10, 180, 22]);
    
    S.listChannels = uicontrol('Style', 'popup',...
        'String', channelsStr,...
        'Position', [185, 10, 120, 22]);  
    
    S.pb = uicontrol('style','push',...
                 'unit','pix',...
                 'position',[335 10 100 22],...
                 'string','Apply',...
                 'callback',{@pb_call,S});
             
    uiwait(S.fh)  % Prevent all other processes from starting until closed.

    function [] = pb_call(varargin)
    % Callback for the pushbutton.
        chnnl = get(S.listChannels,'Value');
        close(S.fh);  % Closes the GUI, allows the new R to be returned.
    end

end


function [intensity_threshold, max_xyz_threshold, min_xyz_threshold, top_layer, model_type, circle_data] = getGuiValues(S,mz)
    
    % get threshold values
        intensity_threshold = get(S.sl_intensity,'val');
        max_z_threshold = ceil(get(S.sl_z_max_threshold,'val'));
        min_z_threshold = ceil(get(S.sl_z_min_threshold,'val'));
    % normalize the threshold values to correct oreintation
        max_z_threshold =  mz - max_z_threshold + 1;
        min_z_threshold = mz - min_z_threshold + 1; 

        temp_min = min([max_z_threshold min_z_threshold]);
        max_z_threshold = max([max_z_threshold min_z_threshold]);
        min_z_threshold = temp_min;
        
        max_y_threshold = ceil(get(S.sl_y_max_threshold,'val'));
        min_y_threshold = ceil(get(S.sl_y_min_threshold,'val'));

        temp_min = min([max_y_threshold min_y_threshold]);
        max_y_threshold = max([max_y_threshold min_y_threshold]);
        min_y_threshold = temp_min;
        
        max_x_threshold = ceil(get(S.sl_x_max_threshold,'val'));
        min_x_threshold = ceil(get(S.sl_x_min_threshold,'val'));
        
        temp_min = min([max_x_threshold min_x_threshold]);
        max_x_threshold = max([max_x_threshold min_x_threshold]);
        min_x_threshold = temp_min;
        
        max_xyz_threshold = [max_x_threshold max_y_threshold max_z_threshold];
        min_xyz_threshold = [min_x_threshold min_y_threshold min_z_threshold];
        
        
        if S.r1.Value == 1 
            top_layer = 1;
        else
            top_layer = 0;
        end
        
        if S.r_strict.Value == 1
            model_type = 1;
        elseif S.r_strict2.Value == 1
            model_type = 2;
        else
            model_type = 3;
        end
        
        circle_data = [];
        if ~isempty(S.xc.String)
            circle_data(1) = str2num(S.xc.String);
            circle_data(2) = str2num(S.zc.String);
            circle_data(3) = str2num(S.radius.String);
        end
        
        
end


function [models] = promptModelSurface(vImarisApplication)
        
	% declares parameters for the distance transform
    	vImarisDataSet = vImarisApplication.GetDataSet;
        data = vImarisDataSet.GetDataShorts;
        
        data = permute(data,[4 3 5 2 1]); % data = permute(data,[3 4 5 2 1]); %CHANGED CHANGED       
        
        %save('C:\Users\Topham_Lab\Desktop\Model_Surfaces_Get_Distances\data\data.mat','data');

        %[intensity_threshold, max_z_threshold, min_z_threshold, chosen_channel, top_layer] = threshold_intensity_and_depth(data,1);
        [intensity_threshold, max_xyz_threshold, min_xyz_threshold, chosen_channel, top_layer, model_type, circle_data] = threshold_intensity_and_depth_xyz(vImarisApplication,data,1)
        
%         intensity_threshold = 631.8417;
%         max_xyz_threshold = [512   256    71];
%         min_xyz_threshold = [1     1    37];
%         chosen_channel = 3;
%         top_layer = 0;
%         model_type = 3;
%         circle_data = [426.5    4151.2    4193.2];

%           blue image
%         intensity_threshold = 473;
%         max_xyz_threshold = [256   256    30];
%         min_xyz_threshold = [1     1     1];
%         chosen_channel = 1;
%         top_layer = 1;
%         model_type = 1;
%         circle_data = [];

        [ModelSurface, model_xyzs, layer_type] = MakeModelSurface(vImarisApplication, data, intensity_threshold, max_xyz_threshold, min_xyz_threshold, chosen_channel, top_layer, model_type, circle_data);
        
        models = struct();
        models(1).ModelSurface = ModelSurface;
        models(1).model_xyzs = model_xyzs;
        models(1).layer_type = layer_type;
        models(1).chosen_channel = chosen_channel;
end


function [intensity_threshold, max_xyz_threshold, min_xyz_threshold, chosen_channel, top_layer, model_type, circle_data] = threshold_intensity_and_depth_xyz(vImarisApplication, data, layer, channel, varargin)
%
%   gui_threshold will get the z-axis and intensity threshold to pinpoint
%   the image signal to model
%
    vImarisDataSet = vImarisApplication.GetDataSet;
    vDataMin = [vImarisDataSet.GetExtendMinX, vImarisDataSet.GetExtendMinY, vImarisDataSet.GetExtendMinZ];
    vDataMax = [vImarisDataSet.GetExtendMaxX, vImarisDataSet.GetExtendMaxY, vImarisDataSet.GetExtendMaxZ];
    vDataSize = [vImarisDataSet.GetSizeX, vImarisDataSet.GetSizeY, vImarisDataSet.GetSizeZ];
    pixelSizeZ = (vDataMax(3) - vDataMin(3)) / vDataSize(3);
    pixelSizeY = (vDataMax(2) - vDataMin(2)) / vDataSize(2);
    pixelSizeX = (vDataMax(1) - vDataMin(1)) / vDataSize(1);


    if nargin == 4
        top_layer = layer;
        chosen_channel = channel;
    elseif nargin == 3
        chosen_channel = choose_channel(data);
        top_layer = layer;
    else
        chosen_channel = choose_channel(data);
        top_layer = 1;
    end
%     chosen_channel = choose_channel_2(data);
%     chosen_channel = 1;

    % graph image with default thresholds
    [mx,my,mz,mc,mt]=size(data);
    cmap=[1:mc];
    background=mean(data(:,:,:,chosen_channel,:),5);
    min_intensity = min(min(min(min(min(data)))));
    max_intensity = max(max(max(max(max(data)))));
    f = figure;
    intensity_threshold = mean([min_intensity max_intensity]);
%    intensity_threshold = 600;
    max_z_threshold = mz;
    min_z_threshold = 1;
    [xxx,yyy,zzz]=meshgrid([1:my],[1:mx],fliplr([1:mz]));
    bwbackground = background > intensity_threshold;
    

    if top_layer
        for i=1:mx
            for j=1:my
                k=find(bwbackground(i,j,:)>0,1);
                bwbackground(i,j,k+1:end)=0;
            end
        end
    else
        for i=1:mx
            for j=1:my
                k=find(bwbackground(i,j,:)>0);
                maxk = max(k);
                bwbackground(i,j,1:maxk-1)=0;
            end
        end     
    end
        

    
    isosurface(xxx,yyy,zzz,bwbackground,[.9])
    axis ij;
    axis equal;  
    zlabel('z')
    ylabel('x')
    xlabel('y')

    %S = makeThresholdGui(min_intensity, intensity_threshold, max_intensity, mz);
    % Create threshold GUI
        S.fh = figure('units','pixels',...
                'position',[400 400 700 350],...
                'menubar','none',...
                'name','Choose intensity_threshold',...
                'numbertitle','off',...
                'resize','off',...
                'deletefcn',{@delete_clean,f});

    % Create Sliders     
        S.sl_intensity = uicontrol('style','slide',...
                    'unit','pixel',...
                    'String','intensity_threshold',...
                    'position',[10 30 350 20],...
                    'min',min_intensity,'value',intensity_threshold,'max',max_intensity,...
                    'callback',{@sl_call,background,f});

        S.sl_z_max_threshold = uicontrol('style','slide',...
                    'unit','pixel',...
                    'String','max_z_threshold',...
                    'position',[400 10 30 200],...
                    'min',1,'value',mz,'max',mz,...
                    'callback',{@sl_call,background,f});

        S.sl_z_min_threshold = uicontrol('style','slide',...
                    'unit','pixel',...
                    'String','min_z_threshold',...
                    'position',[430 10 30 200],...
                    'min',1,'value',1,'max',mz,...
                    'callback',{@sl_call,background,f});

        S.sl_x_min_threshold = uicontrol('style','slide',...
                    'unit','pixel',...
                    'String','min_z_threshold',...
                    'position',[480 140 200 30],...
                    'min',1,'value',1,'max',mx,...
                    'callback',{@sl_call,background,f});

        S.sl_x_max_threshold = uicontrol('style','slide',...
                    'unit','pixel',...
                    'String','min_z_threshold',...
                    'position',[480 100 200 30],...
                    'min',1,'value',mx,'max',mx,...
                    'callback',{@sl_call,background,f});

        S.sl_y_min_threshold = uicontrol('style','slide',...
                    'unit','pixel',...
                    'String','min_z_threshold',...
                    'position',[480 50 200 30],...
                    'min',1,'value',1,'max',my,...
                    'callback',{@sl_call,background,f});

        S.sl_y_max_threshold = uicontrol('style','slide',...
                    'unit','pixel',...
                    'String','min_z_threshold',...
                    'position',[480 10 200 30],...
                    'min',1,'value',my,'max',my,...
                    'callback',{@sl_call,background,f});


    % Apply Button         
        S.button = uicontrol('style','pushbutton',...
                    'unit','pixel',...
                    'position',[160 5 60 25],...
                    'fontsize',12,...
                    'string','Apply',...
                    'callback',{@apply_call,f});   
       
        S.testModel = uicontrol('style','pushbutton',...
                    'unit','pixel',...
                    'position',[250 5 60 25],...
                    'fontsize',12,...
                    'string','Test',...
                    'callback',{@test_model,background,f});

    
    % radial buttons
        S.bg = uibuttongroup(S.fh,'Visible','on',...
                    'unit','pixel',...
                    'Position',[5 245 140 100]);

        S.r1 = uicontrol(S.bg,'Style','radiobutton',...
                    'String','Get Top Layer',...
                    'unit','pixel',...
                    'Position',[5 60 120 30],...
                    'HandleVisibility','on',...
                    'callback',{@sl_call,background,f});              

        S.r2 = uicontrol(S.bg,'Style','radiobutton',...
                    'String','Get Bottom Layer',...
                    'unit','pixel',...
                    'Position',[5 30 120 30],...
                    'HandleVisibility','on',...
                    'callback',{@sl_call,background,f});


        S.bg2 = uibuttongroup(S.fh,'Visible','on',...
                      'unit','pixel',...
                      'Position',[160 125 210 220]);

        S.r_strict = uicontrol(S.bg2,'Style','radiobutton',...
                      'String','Strict Model (High Signal)',...
                      'unit','pixel',...
                      'Position',[5 185 180 30],...
                      'HandleVisibility','on',...
                      'callback',{@is_circle,background,f});    

        S.r_strict2 = uicontrol(S.bg2,'Style','radiobutton',...
                      'String','Loose Model (Low Signal)',...
                      'unit','pixel',...
                      'Position',[5 155 180 30],...
                      'HandleVisibility','on',...
                      'callback',{@is_circle,background,f}); 
                  
        S.r_strict3 = uicontrol(S.bg2,'Style','radiobutton',...
                      'String','Circle Model',...
                      'unit','pixel',...
                      'Position',[5 125 180 30],...
                      'HandleVisibility','on',...
                      'callback',{@is_circle,background,f}); 
                  
        S.radius = uicontrol(S.bg2,'Style','Edit',...
                      'String','',...
                      'unit','pixel',...
                      'Position',[55 95 140 20],...
                      'Visible','off');
        S.xc = uicontrol(S.bg2,'Style','Edit',...
                    'String','',...
                    'unit','pixel',...
                    'Position',[55 65 140 20],...
                    'Visible','off');
        S.zc = uicontrol(S.bg2,'Style','Edit',...
                    'String','',...
                    'unit','pixel',...
                    'Position',[55 35 140 20],...
                    'Visible','off');
                
                
        S.radiusLabel = uicontrol(S.bg2,'Style','Text',...
                    'String','Radius:',...
                    'unit','pixel',...
                    'Position',[1 95 50 20],...
                    'Visible','off');
        S.xcLabel = uicontrol(S.bg2,'Style','Text',...
                    'String','Center X:',...
                    'unit','pixel',...
                    'Position',[1 65 50 20],...
                    'Visible','off');
        S.zcLabel = uicontrol(S.bg2,'Style','Text',...
                    'String','Center Z:',...
                    'unit','pixel',...
                    'Position',[1 35 50 20],...
                    'Visible','off');
        S.reset = uicontrol(S.bg2,'Style','push',...
                    'String','Reset',...
                    'unit','pixel',...
                    'Position',[55 5 140 20],...
                    'Visible','off',...
                    'callback',{@reset_circle});

            
            
    %% Add slider labels
        S.text = uicontrol('style','text',...
                    'unit','pixel',...
                    'position',[10 55 200 30],...
                    'fontsize',12,...
                    'string',num2str(intensity_threshold)); 

        S.text2 = uicontrol('style','text',...
                    'unit','pixel',...
                    'position',[10 85 200 30],...
                    'fontsize',12,...
                    'string','Set Intensity Theshold:'); 

        S.text3 = uicontrol('style','text',...
                    'unit','pixel',...
                    'String','Set z-axis:',...
                    'position',[380 213 100 20],...
                    'fontsize',12);

        S.text_y = uicontrol('style','text',...
                    'unit','pixel',...
                    'String','Set y-axis:',...
                    'position',[470 80 200 20],...
                    'fontsize',12);

        S.text_x = uicontrol('style','text',...
                    'unit','pixel',...
                    'String','Set x-axis:',...
                    'position',[470 170 200 20],...
                    'fontsize',12);

            
    set(f,'deletefcn',{@delete_clean,S.fh})  % Closing one closes the other.
    

    %%
    function [intensity_threshold, max_z_threshold, min_z_threshold] = sl_call(varargin) 
    % Callback for the slider.
        [h,background,f] = deal(varargin{[1;3;4;]});  % Get the calling handle and structure.
        
        % get dimensions of image
        [mx,my,mz,mc,mt]=size(background);

        % erase existing figure and prime it for new graph
        f.clo
        figure(f)
        
        % get threshold values
        intensity_threshold = get(S.sl_intensity,'val');
        max_z_threshold = ceil(get(S.sl_z_max_threshold,'val'));
        min_z_threshold = ceil(get(S.sl_z_min_threshold,'val'));
        % normalize the threshold values to correct oreintation
        max_z_threshold =  mz - max_z_threshold + 1;
        min_z_threshold = mz - min_z_threshold + 1; 
        % check if min is less than max threshold. If not, switch values
        temp_min = min([max_z_threshold min_z_threshold]);
        max_z_threshold = max([max_z_threshold min_z_threshold]);
        min_z_threshold = temp_min;
        
        max_y_threshold = ceil(get(S.sl_y_max_threshold,'val'));
        min_y_threshold = ceil(get(S.sl_y_min_threshold,'val'));
        % normalize the threshold values to correct oreintation
%         max_y_threshold =  my - max_y_threshold + 1;
%         min_y_threshold = my - min_y_threshold + 1; 
        % check if min is less than max threshold. If not, switch values
        temp_min = min([max_y_threshold min_y_threshold]);
        max_y_threshold = max([max_y_threshold min_y_threshold]);
        min_y_threshold = temp_min;
        
        max_x_threshold = ceil(get(S.sl_x_max_threshold,'val'));
        min_x_threshold = ceil(get(S.sl_x_min_threshold,'val'));
        % normalize the threshold values to correct oreintation
%         max_x_threshold =  mx - max_x_threshold + 1;
%         min_x_threshold = mx - min_x_threshold + 1; 
        % check if min is less than max threshold. If not, switch values
        temp_min = min([max_x_threshold min_x_threshold]);
        max_x_threshold = max([max_x_threshold min_x_threshold]);
        min_x_threshold = temp_min;
        
        

%         disp('mz')
%         disp(mz)
%         disp('min_x_threshold')
%         disp(min_x_threshold)
%         disp('max_x_threshold')
%         disp(max_x_threshold)
        
        % make isosurface using threshold values
        temp = zeros(size(background));
        temp(min_x_threshold:max_x_threshold,min_y_threshold:max_y_threshold,(min_z_threshold:max_z_threshold)) = background(min_x_threshold:max_x_threshold,min_y_threshold:max_y_threshold,(min_z_threshold:max_z_threshold));
        background  = temp;
        [xxx,yyy,zzz]=meshgrid([1:my],[1:mx],fliplr([1:mz]));
        
        
        set(S.text,'string',num2str(intensity_threshold));
        bwbackground = background > intensity_threshold;
        
        if S.r1.Value == 1 
            top_layer = 1;
        else
            top_layer = 0;
        end
        
        if top_layer
            for i=1:mx
                for j=1:my
                    k=find(bwbackground(i,j,:)>0,1);
                    bwbackground(i,j,k+1:end)=0;
                end
            end
        else
            for i=1:mx
                for j=1:my
                    k=find(bwbackground(i,j,:)>0);
                    maxk = max(k);
                    bwbackground(i,j,1:maxk-1)=0;
                end
            end     
        end
        
        
        
        disp('computing...');
        isosurface(xxx,yyy,zzz,bwbackground,[.9]);
        disp('computed');
        axis ij;
        axis equal;
        zlabel('z')
        ylabel('x')
        xlabel('y')
    end


    function [] = apply_call(varargin)
        % Callback for the pushbutton.
            [intensity_threshold, max_xyz_threshold, min_xyz_threshold, top_layer, model_type, circle_data] = getGuiValues(S,mz);
            delete(S.fh);  % Closes the GUI, allows the new R to be returned.
            delete(f);
    end


    function [] = delete_clean(varargin)
        [intensity_threshold, max_xyz_threshold, min_xyz_threshold, top_layer, model_type, circle_data] = getGuiValues(S,mz);
        delete(S.fh);  % Closes the GUI, allows the new R to be returned.
        delete(f);
    end


    function test_model(varargin)

        [intensity_threshold, max_xyz_threshold, min_xyz_threshold, top_layer, model_type, circle_data] = getGuiValues(S,mz);
        
        [mask,circle_data_out] = MakeModelSurface_Single(vImarisApplication, data, intensity_threshold, max_xyz_threshold, min_xyz_threshold, chosen_channel, top_layer, model_type, circle_data);
        
       
        if ~isempty(circle_data_out)
            S.xc.String = num2str(circle_data_out(1));
            S.zc.String = num2str(circle_data_out(2));
            S.radius.String = num2str(circle_data_out(3));
        end
    end


    function is_circle(varargin)
        
        if S.r_strict3.Value == 1
            S.xc.Visible = 'on';
            S.zc.Visible = 'on';
            S.radius.Visible = 'on';
            S.xcLabel.Visible = 'on';
            S.zcLabel.Visible = 'on';
            S.radiusLabel.Visible = 'on';
            S.reset.Visible = 'on';
            if strcmp(S.r_strict3.String, '')
                
            end
        else
            S.xc.Visible = 'off';
            S.zc.Visible = 'off';
            S.radius.Visible = 'off';
            S.xcLabel.Visible = 'off';
            S.zcLabel.Visible = 'off';
            S.radiusLabel.Visible = 'off';
            S.reset.Visible = 'on';
        end   
       
    end


    function reset_circle(varargin)
            S.xc.String = '';
            S.zc.String = '';
            S.radius.String = '';
    end


    uiwait(S.fh)
    
end


function [model_xyz, circle_data_out] = MakeModelSurface_Single(vImarisApplication, data, intensity_threshold, max_xyz_threshold, min_xyz_threshold, chosen_channel, top_layer, model_type, circle_data)
%
% 	FitSurface fits a polynomial surface to a mask. The algorithm used to fit
% 	the surface is fit( [x, y], z, 'poly23' ), a function from the "Curve
% 	Fitting Toolbox" in MATLAB. The fitted surface produced is made into an
% 	Imaris Surface item and added to the scene. Some of this code is
% 	adopted from Henry Pinkard's Imaris XTension "Minimum distance of spots
% 	to surface edge".
% 	(http://open.bitplane.com/tabid/235/Default.aspx?id=89)
%
%		@param vImarisApplication - Imaris application index number
%		@param RefSurface - Imaris Surface object used to fit the
%		surface
%
%
	% declares parameters used in model
    	vImarisDataSet = vImarisApplication.GetDataSet;
		vDataMin = [vImarisDataSet.GetExtendMinX, vImarisDataSet.GetExtendMinY, vImarisDataSet.GetExtendMinZ];
		vDataMax = [vImarisDataSet.GetExtendMaxX, vImarisDataSet.GetExtendMaxY, vImarisDataSet.GetExtendMaxZ];
		vDataSize = [vImarisDataSet.GetSizeX, vImarisDataSet.GetSizeY, vImarisDataSet.GetSizeZ];
		pixelSizeZ = (vDataMax(3) - vDataMin(3)) / vDataSize(3);
		pixelSizeY = (vDataMax(2) - vDataMin(2)) / vDataSize(2);
		pixelSizeX = (vDataMax(1) - vDataMin(1)) / vDataSize(1);
        [mx,my,mz,~,mt]=size(data);

    % create new surface
        newSurface = vImarisApplication.GetFactory.CreateSurfaces();

    % declare output
        circle_data_out = [];
        
    % apply thresholds to 5d data array
        temp = zeros(size(data));
        temp(min_xyz_threshold(1):max_xyz_threshold(1),min_xyz_threshold(2):max_xyz_threshold(2),min_xyz_threshold(3):max_xyz_threshold(3),:,:) = data(min_xyz_threshold(1):max_xyz_threshold(1),min_xyz_threshold(2):max_xyz_threshold(2),min_xyz_threshold(3):max_xyz_threshold(3),:,:);
        data = temp;
        clear temp


    % iterate through time points 
        for timeIndex = 1:1
%        for timeIndex = 1:mt   
            disp([ 'Test Modelling step ', num2str(timeIndex), ' of 1' ])

        %filter using intensity threshold
            bwbackground = data(:,:,:,chosen_channel,timeIndex) > intensity_threshold;

        % since 2nd harmonic should be topmost layer, filter for these values
        % to get shell of layer and remove noise           
            if top_layer   
                layer_type = 'Upper';
                for i=1:mx
                    for j=1:my
                        k=find(bwbackground(i,j,:)>0,1);
                        bwbackground(i,j,k+1:end)=0;
                    end
                end
            else
                layer_type = 'Lower';
                for i=1:mx
                    for j=1:my
                        k=find(bwbackground(i,j,:)>0);
                        maxk = max(k);
                        bwbackground(i,j,1:maxk-1)=0;
                    end
                end     
            end
            
            
             %save('C:\Users\Topham_Lab\Desktop\Model_Surfaces_Get_Distances\data\bwbackground.mat','bwbackground');
            
       % fit the model and add to new channel
            [xx,yy,zz]=ind2sub(size(bwbackground),find(bwbackground)); 
            [xData, yData, zData] = prepareSurfaceData( yy, xx, zz ); %% changed xx and yy
            
            if model_type == 1
                ft = fittype( 'poly44' );
            else
                ft = fittype( 'poly22' );       
            end
            
            [fitresult] = fit( [xData, yData], zData, ft );
 
            ix = 0;          
            for i = 1:vDataSize(1) % reversed x and y
                for j = 1:vDataSize(2)
                    %z_val = round((fitresult(i,j)));
                    z_val = (fitresult(i,j));
                    if z_val < 1
                        z_ind = 1;
                    elseif z_val > mz
                        z_ind = mz;
                    else
                        z_ind = z_val;
                    end
                    ix = ix + 1;
                    x(ix) = i;
                    y(ix) = j;
                    z(ix) = z_val;
                end
            end
            model_xyz =  horzcat(x',y',z');

            if model_type == 3
                
                [xData, yData, zData] = prepareSurfaceData( y, x, z ); %% changed xx and yy 



                if isempty(circle_data)
                    [xc,zc,r,~] = circfit(xData,zData);
                else
                    xc = circle_data(1);
                    zc = circle_data(2);
                    r = circle_data(3);
                end
                
                circle_data_out(1) = xc;
                circle_data_out(2) = zc;
                circle_data_out(3) = r;
                

                ang=0:0.01:2*pi;
                xp=r*cos(ang);
                zp=r*sin(ang);
                f2 = figure;
                plot(xData,zData,'o',xc+xp,zc+zp,'-.'), title(' measured points')
                close(f2);
                
                for i = 1:mx
                    ang = acos((i-xc)/r);
                    x_circle(i) = i;
                    z_circle(i) = real(r*sin(-ang)+zc);
                end

                y = repmat(x_circle,1,my);
                z = repmat(z_circle,1,my);
                x = repmat(1:1:my,length(x_circle),1);
                x = x(:)';   
                
                model_xyz = horzcat(x',y',z');
                    
                   
            end
            
            % normalize the coords to work with zero indexed imaris
                x = x - 1;
                y = y - 1;
                z = z - 1;
            
            % get image data of the surface, vertices, triangles and normals
                tri = delaunay(x,y);
                tr = triangulation(tri,x(:),y(:),z(:));
                f = figure;
                t = trisurf(tr);
                vVertices = t.Vertices;
                vTriangles = t.Faces-1;
                vNormals = vertexNormal(tr);
                close(f)

            % scale the image
                vVertices(:,1) = vVertices(:,1) * pixelSizeX; 
                vVertices(:,2) = vVertices(:,2) * pixelSizeY; 
                vVertices(:,3) = vVertices(:,3) * pixelSizeZ;

            % add data to newSurface for timeIndex
                newSurface.AddSurface(vVertices, vTriangles, vNormals, timeIndex-1); 
                
                model_xyzs{timeIndex} = model_xyz;
                %masks{timeIndex} = mask;

        end
        


    fprintf('\tTest Model Complete\n')	
    % prepare parameters for Imaris Surface item
        vRed = 0.9;vGreen = 0.1;vBlue = 0.3;vAlpha = 0.0;vRGBA = [vRed, vGreen, vBlue, vAlpha];
        vRGBA = round(vRGBA * 255);vRGBA = uint32(vRGBA * [1; 256; 256*256; 256*256*256]); % combine different components (four bytes) into one integer
        newSurface.SetColorRGBA (vRGBA);
        newSurface.SetName (['Modeled Surface Test ' layer_type ' Timepoint 1']);
        newSurface.SetVisible (1);

    % add Imaris Surface item to Scene
        vSurpassScene = vImarisApplication.GetSurpassScene;
        vSurpassScene.AddChild(newSurface, -1);
        vDataItem = vSurpassScene.GetChild(vSurpassScene.GetNumberOfChildren-1);

    % return the Fitted Surface
        vImarisApplication.GetFactory.ToSurfaces(vDataItem);            


end


function [ModelSurface, model_xyzs, layer_type] = MakeModelSurface(vImarisApplication, data, intensity_threshold, max_xyz_threshold, min_xyz_threshold, chosen_channel, top_layer, model_type, circle_data)
%
% 	FitSurface fits a polynomial surface to a mask. The algorithm used to fit
% 	the surface is fit( [x, y], z, 'poly23' ), a function from the "Curve
% 	Fitting Toolbox" in MATLAB. The fitted surface produced is made into an
% 	Imaris Surface item and added to the scene. Some of this code is
% 	adopted from Henry Pinkard's Imaris XTension "Minimum distance of spots
% 	to surface edge".
% 	(http://open.bitplane.com/tabid/235/Default.aspx?id=89)
%
%		@param vImarisApplication - Imaris application index number
%		@param RefSurface - Imaris Surface object used to fit the
%		surface
%
%
	% declares parameters used in model
    	vImarisDataSet = vImarisApplication.GetDataSet;
		vDataMin = [vImarisDataSet.GetExtendMinX, vImarisDataSet.GetExtendMinY, vImarisDataSet.GetExtendMinZ];
		vDataMax = [vImarisDataSet.GetExtendMaxX, vImarisDataSet.GetExtendMaxY, vImarisDataSet.GetExtendMaxZ];
		vDataSize = [vImarisDataSet.GetSizeX, vImarisDataSet.GetSizeY, vImarisDataSet.GetSizeZ];
		pixelSizeZ = (vDataMax(3) - vDataMin(3)) / vDataSize(3);
		pixelSizeY = (vDataMax(2) - vDataMin(2)) / vDataSize(2);
		pixelSizeX = (vDataMax(1) - vDataMin(1)) / vDataSize(1);
        [mx,my,mz,~,mt]=size(data);

    % create new surface
        newSurface = vImarisApplication.GetFactory.CreateSurfaces();

    % apply thresholds to 5d data array
        temp = zeros(size(data));
        temp(min_xyz_threshold(1):max_xyz_threshold(1),min_xyz_threshold(2):max_xyz_threshold(2),min_xyz_threshold(3):max_xyz_threshold(3),:,:) = data(min_xyz_threshold(1):max_xyz_threshold(1),min_xyz_threshold(2):max_xyz_threshold(2),min_xyz_threshold(3):max_xyz_threshold(3),:,:);
        data = temp;
        clear temp

    % iterate through time points 
%        for timeIndex = 1:1
        for timeIndex = 1:mt   
            disp([ 'Modelling step ', num2str(timeIndex), ' of ', num2str(mt) ])

        %filter using intensity threshold
            bwbackground = data(:,:,:,chosen_channel,timeIndex) > intensity_threshold;

        % since 2nd harmonic should be topmost layer, filter for these values
        % to get shell of layer and remove noise           
            if top_layer   
                layer_type = 'Upper';
                for i=1:mx
                    for j=1:my
                        k=find(bwbackground(i,j,:)>0,1);
                        bwbackground(i,j,k+1:end)=0;
                    end
                end
            else
                layer_type = 'Lower';
                for i=1:mx
                    for j=1:my
                        k=find(bwbackground(i,j,:)>0);
                        maxk = max(k);
                        bwbackground(i,j,1:maxk-1)=0;
                    end
                end     
            end
            
            
             %save('C:\Users\Topham_Lab\Desktop\Model_Surfaces_Get_Distances\data\bwbackground.mat','bwbackground');
            
       % fit the model and add to new channel
            [xx,yy,zz]=ind2sub(size(bwbackground),find(bwbackground)); 
            [xData, yData, zData] = prepareSurfaceData( yy, xx, zz ); %% changed xx and yy
            
            if model_type == 1
                ft = fittype( 'poly44' );
            else
                ft = fittype( 'poly22' );       
            end
            
            [fitresult] = fit( [xData, yData], zData, ft );
 
            ix = 0;          
            for i = 1:vDataSize(1) % reversed x and y
                for j = 1:vDataSize(2)
                    %z_val = round((fitresult(i,j)));
                    z_val = (fitresult(i,j));
                    if z_val < 1
                        z_ind = 1;
                    elseif z_val > mz
                        z_ind = mz;
                    else
                        z_ind = z_val;
                    end
                    ix = ix + 1;
                    x(ix) = i;
                    y(ix) = j;
                    z(ix) = z_val;
                end
            end
            
            model_xyz = horzcat(x',y',z');

            % if circle need to take data through another round of modeling
            if model_type == 3
                
                [xData, ~, zData] = prepareSurfaceData( y, x, z ); %% changed xx and yy 

                if isempty(circle_data)
                    [xc,zc,r,~] = circfit(xData,zData);
                    circle_data(1) = xc;
                    circle_data(2) = zc;
                    circle_data(3) = r;
                else
                    xc = circle_data(1);
                    zc = circle_data(2);
                    r = circle_data(3);
                end

                ang=0:0.01:2*pi;
                xp=r*cos(ang);
                zp=r*sin(ang);
                f2 = figure;
                plot(xData,zData,'o',xc+xp,zc+zp,'-.'), title(' measured points')
                close(f2);
                
                x_circle = zeros(1,mx);
                z_circle = zeros(1,mx);
                for i = 1:mx
                    ang = acos((i-xc)/r);
                    x_circle(i) = i;
                    %z_circle(i) = round(r*sin(-ang)+zc);
                    z_circle(i) = real(r*sin(-ang)+zc);
                end

                y = repmat(x_circle,1,my);
                z = repmat(z_circle,1,my);
                x = repmat(1:1:my,length(x_circle),1);
                x = x(:)';   
                
                model_xyz = horzcat(x',y',z');  
            end
            
            % normalize the coords to work with zero indexed imaris
                x = x - 1;
                y = y - 1;
                z = z - 1;
            
            % get image data of the surface, vertices, triangles and normals
                tri = delaunay(x,y);
                tr = triangulation(tri,x(:),y(:),z(:));
                f = figure;
                t = trisurf(tr);
                vVertices = t.Vertices;
                vTriangles = t.Faces-1;
                vNormals = vertexNormal(tr);

            % scale the image
                vVertices(:,1) = vVertices(:,1) * pixelSizeX; 
                vVertices(:,2) = vVertices(:,2) * pixelSizeY; 
                vVertices(:,3) = vVertices(:,3) * pixelSizeZ;
                close(f)

            % add data to newSurface for timeIndex
                newSurface.AddSurface(vVertices, vTriangles, vNormals, timeIndex-1); 
                
                model_xyzs{timeIndex} = model_xyz;
                %masks{timeIndex} = mask;

        end
        

    % prepare parameters for Imaris Surface item
        vRed = 0.9;vGreen = 0.1;vBlue = 0.3;vAlpha = 0.0;vRGBA = [vRed, vGreen, vBlue, vAlpha];
        vRGBA = round(vRGBA * 255);vRGBA = uint32(vRGBA * [1; 256; 256*256; 256*256*256]); % combine different components (four bytes) into one integer
        newSurface.SetColorRGBA (vRGBA);
        newSurface.SetName (['Modeled Surface ' layer_type ]);
        newSurface.SetVisible (1);

    % add Imaris Surface item to Scene
        vSurpassScene = vImarisApplication.GetSurpassScene;
        vSurpassScene.AddChild(newSurface, -1);
        vDataItem = vSurpassScene.GetChild(vSurpassScene.GetNumberOfChildren-1);

    % return the Fitted Surface
        ModelSurface = vImarisApplication.GetFactory.ToSurfaces(vDataItem);            

        fprintf('\tModel Complete\n')	

end


function getDistancesFromModeledSurf(vImarisApplication, CellSurfaces, model_xyzs, layer_type)

	
   	% preparing parameters for the distance transform
 		vImarisDataSet = vImarisApplication.GetDataSet;
		vDataMin = [vImarisDataSet.GetExtendMinX, vImarisDataSet.GetExtendMinY, vImarisDataSet.GetExtendMinZ];
		vDataMax = [vImarisDataSet.GetExtendMaxX, vImarisDataSet.GetExtendMaxY, vImarisDataSet.GetExtendMaxZ];
		vDataSize = [vImarisDataSet.GetSizeX, vImarisDataSet.GetSizeY, vImarisDataSet.GetSizeZ];
		pixelSizeZ = (vDataMax(3) - vDataMin(3)) / vDataSize(3);
		pixelSizeX = (vDataMax(1) - vDataMin(1)) / vDataSize(1);
		pixelSizeY = (vDataMax(2) - vDataMin(2)) / vDataSize(2);
        aspect = [pixelSizeX pixelSizeY pixelSizeZ];
		numTimePoints = vImarisDataSet.GetSizeT;
              
	% declaring the parameters of the tracked cells
		tracksStruct = getTrackIndicesStruct( CellSurfaces );	
		surfaceTimeIndices = vertcat(tracksStruct.times);
		surfaceXYZ = vertcat(tracksStruct.coords);
		surfaceIds = vertcat(tracksStruct.ids);
        surfaceInds = vertcat(tracksStruct.inds);
		surfaceTrackIDs = vertcat(tracksStruct.TrackIDs);
		numSurfaces = length( surfaceTimeIndices );	
		distances = zeros(numSurfaces,1);
		distancesz = zeros(numSurfaces,1);
	   
        
    % creates Imaris MeasurementPoints item to store distances
        bwdist_data = null(1);
        zdist_data = null(1);
        
        vProgressDisplay = waitbar(0, 'Calculating distances');

	% iterates through the time points to find the distance from reference to each cell through all time points
		for timeIndex = 0:numTimePoints-1;
% 		for timeIndex = 0:0;
        
        % give update to user on steps left in process
			fprintf('\tStep %d of %d\n',timeIndex, numTimePoints-1 );            
            %waitbar((timeIndex+1)/numTimePoints, vProgressDisplay);  
		
		% get the mask DataSet, 8 bit binary data set		
            %mask = masks{timeIndex+1};
            model_xyz = model_xyzs{timeIndex+1};


		% iterate through each surface at this timepoint, calculate its distance
		% from the surface based on the distanceTransform, and add that as a
		% custom statistic to imaris
            inds = find(surfaceTimeIndices == timeIndex).';
			for surfaceIndex = inds
                
				coordinates = surfaceXYZ(surfaceIndex,:);
				indices = ceil((coordinates - vDataMin) ./ [pixelSizeX pixelSizeY pixelSizeZ]);
                
%                 disp('indices')
%                 disp(indices)
                [min_distance,min_coord1,min_coord2] = getMinDistanceFromModelToCoord(model_xyz,coordinates,aspect);

                distances(surfaceIndex) = min_distance;
                 
                %add MeasurementPoint data
                bwdist_data = addMeasurementPointData(bwdist_data, min_coord2, min_coord1, num2str(surfaceIds(surfaceIndex)), ' ', timeIndex, timeIndex);

               
                inds1 = (model_xyz(:,1)-1) == indices(1);
                inds2 = (model_xyz(:,2)-1) == indices(2);
                z_ind = find(inds1 .* inds2);
%                 disp('z_ind')
%                 disp(z_ind)                
                z_pos =  (model_xyz(z_ind,3) - 1) * pixelSizeZ;
%                 disp('z_pos')
%                 disp(z_pos)
%                 
                if( numel(z_ind) > 0 )
                    refPosition = ([coordinates(1) coordinates(2) z_pos]);           
%                     disp('refPosition')
%                     disp(refPosition)
%                     disp('coordinates')
%                     disp(coordinates)
                    %add MeasurementPoint data
                    zdist_data = addMeasurementPointData(zdist_data, coordinates, refPosition, num2str(surfaceIds(surfaceIndex)), ' ', timeIndex, timeIndex);
                    z_distance = sqrt(sum((coordinates - refPosition) .^ 2,2));
                    distancesz(surfaceIndex) = z_distance;
                else
                    disp('none')
                    disp(surfaceIds(surfaceIndex));
                    disp([indices(1) indices(2)])
                    if timeIndex == 0
%                         save('C:\Users\Topham_Lab\Desktop\Registration_Matlab\data\Add_Model\xt\mask.mat', 'mask');
                    end
                end
            end
                   
        end
        
        close(vProgressDisplay);

        % add MeasurementPoints to Surpass Scene
            disp('BWDIST transform...')
            measurementName = ['BWDIST transform ' layer_type];
            setMeasurementPoint(vImarisApplication, measurementName, bwdist_data, 0.9, 0.3)
            disp('BWDIST transform done.')

            disp('Distances Z test...')
            measurementName = ['Distances Z test ' layer_type];
            setMeasurementPoint(vImarisApplication, measurementName, zdist_data, 0.3, 0.9)
            disp('Distances Z test done.')
        
        % create the distance statistic 
            disp('adding stat')
            statName = ['Distance From Modelled Surface ' layer_type];
            AddStats( CellSurfaces, statName, 'um', surfaceTimeIndices, surfaceIds, distances  )
            
            disp('adding mean stat')
            statName = ['Track ' statName];
            addTrackMean( CellSurfaces, statName, 'um', distances, surfaceTrackIDs )
            
        % create the z distance statistic 
            disp('adding stat z distance')
            statName = ['Distance From Modelled Surface ' layer_type ' Z axis'];
            AddStats( CellSurfaces, statName, 'um', surfaceTimeIndices, surfaceIds, distancesz  )
            
            disp('adding mean stat')
            statName = ['Track ' statName];
            addTrackMean( CellSurfaces, statName, 'um', distancesz, surfaceTrackIDs )
            
end


function [min_distance,min_coord1,min_coord2] = getMinDistanceFromModelToCoord(model_xyz,coord,aspect)
    %
    %   fastest distance function
    %
        
%     save('C:\Users\Topham_Lab\Desktop\Model_Surfaces_Get_Distances\xt\model_xyz.mat','model_xyz');
%     save('C:\Users\Topham_Lab\Desktop\Model_Surfaces_Get_Distances\xt\coord.mat','coord');
%     save('C:\Users\Topham_Lab\Desktop\Model_Surfaces_Get_Distances\xt\aspect.mat','aspect');
    % normaliz the coordinates to zero indexed imaris
    model_xyz = model_xyz - 1;
    x = model_xyz(:,1)';
    y = model_xyz(:,2)';
    z = model_xyz(:,3)';
    coord1 = model_xyz .* repmat( aspect, length(x), 1);
    coord2 = repmat( coord, length(x), 1);
    distances = sqrt(sum((coord1 - coord2) .^ 2,2));
    [min_distance, min_indx] = min(distances);
    min_coord1 = coord1(min_indx,:);
    min_coord2 = coord;

end
